@@ -14,8 +14,7 @@ from arrlio.models import Event, Message, TaskInstance, TaskResult
 from arrlio.settings import ENV_PREFIX
 from arrlio.tp import AsyncCallableT, PriorityT, SerializerT
 
-
-logger = logging.getLogger("arrlio")
+logger = logging.getLogger("arrlio.backends.local")
 
 
 BACKEND_NAME: str = "arrlio"
@@ -65,7 +64,7 @@ class Backend(base.Backend):
                 del self.__shared[self.config.name]
 
     def __str__(self):
-        return f"[LocalBackend({self.config.name})]"
+        return f"LocalBackend[{self.config.name}]"
 
     @property
     def _shared(self) -> dict:
@@ -115,20 +114,18 @@ class Backend(base.Backend):
         for queue in queues:
             self._task_consumers[queue] = asyncio.create_task(consume_queue(queue))
 
-    async def stop_consume_tasks(self):
-        for queue in self._task_consumers.keys():
-            self._task_consumers[queue].cancel()
-        self._task_consumers = {}
+    async def stop_consume_tasks(self, queues: List[str] = None):
+        for queue in list(self._task_consumers.keys()):
+            if queues is None or queue in queues:
+                self._task_consumers[queue].cancel()
+                del self._task_consumers[queue]
 
     @base.Backend.task
     async def push_task_result(self, task_instance: TaskInstance, task_result: TaskResult):
         if not task_instance.data.result_return:
             return
         task_id: UUID = task_instance.data.task_id
-        self._results[task_id][1] = self.serializer.dumps_task_result(
-            task_result,
-            encrypt=task_instance.data.result_encrypt,
-        )
+        self._results[task_id][1] = self.serializer.dumps_task_result(task_instance, task_result)
         self._results[task_id][0].set()
         if task_instance.data.result_ttl is not None:
             loop = asyncio.get_event_loop()
@@ -152,7 +149,7 @@ class Backend(base.Backend):
     @base.Backend.task
     async def send_message(self, message: Message, **kwds):
         data = dataclasses.asdict(message)
-        data["data"] = self.serializer.dumps(message.data, encrypt=message.encrypt)
+        data["data"] = self.serializer.dumps(message.data)
         logger.debug("%s: put %s", self, message)
         await self._message_queues[message.exchange].put(
             (
@@ -201,6 +198,9 @@ class Backend(base.Backend):
 
     @base.Backend.task
     async def consume_events(self, on_event: AsyncCallableT):
+        if self._events_consumer:
+            raise Exception("Already consuming")
+
         async def consume():
             logger.info("%s: consuming events", self)
             while True:
