@@ -19,8 +19,7 @@ from arrlio.settings import ENV_PREFIX
 from arrlio.tp import AsyncCallableT, ExceptionFilterT, PositiveIntT, PriorityT, RMQDsn, SerializerT, TimeoutT
 from arrlio.utils import retry
 
-
-logger = logging.getLogger("arrlio")
+logger = logging.getLogger("arrlio.backends.rabbitmq")
 
 
 BACKEND_NAME: str = "arrlio"
@@ -173,7 +172,7 @@ class RMQConnection:
                 else:
                     callback()
             except Exception as e:
-                logger.error("Callback '%s' %s error: %s %s", tp, callback, e.__class__, e)
+                logger.error("%s: callback '%s' %s error: %s %s", self, tp, callback, e.__class__, e)
 
     async def _supervisor(self):
         try:
@@ -183,6 +182,7 @@ class RMQConnection:
         if not self._closed.done():
             logger.warning("%s: connection lost", self)
             self._refs -= 1
+            self._supervisor_task = None
             await self._execute_callbacks("on_lost")
 
     async def open(self):
@@ -275,7 +275,7 @@ class Backend(base.Backend):
             logger.warning("Unclosed %s", self)
 
     def __str__(self):
-        return f"[RMQBackend[{self.__conn}]]"
+        return f"RMQBackend[{self.__conn}]"
 
     @property
     def _conn(self):
@@ -349,7 +349,7 @@ class Backend(base.Backend):
     @base.Backend.task
     async def send_task(self, task_instance: TaskInstance, result_queue_durable: bool = False, **kwds):
         task_data = task_instance.data
-        task_data.backend_extra["result_queue_durable"] = result_queue_durable
+        task_data.extra["result_queue_durable"] = result_queue_durable
         await self.declare_task_queue(task_data.queue)
         logger.debug("%s: put %s", self, task_instance)
         async with self._conn.channel_ctx() as channel:
@@ -382,8 +382,8 @@ class Backend(base.Backend):
                     await asyncio.shield(on_task(task_instance))
                     if ack_late:
                         await channel.basic_ack(msg.delivery.delivery_tag)
-                except Exception:
-                    logger.exception("Internal error")
+                except Exception as e:
+                    logger.exception(e)
 
             for queue in queues:
                 if queue in self._task_consumers and not self._task_consumers[queue][0].is_closed:
@@ -402,23 +402,29 @@ class Backend(base.Backend):
 
             self._conn.add_callback("on_lost", "consume_tasks", _consume_tasks)
 
-    async def stop_consume_tasks(self):
-        self.__conn.remove_callback("on_lost", "consume_tasks")
-        self.__conn.remove_callback("on_close", "consume_tasks")
+    async def stop_consume_tasks(self, queues: List[str] = None):
         async with self._consume_lock:
-            for queue, (channel, consume_ok) in self._task_consumers.items():
-                if not self.__conn.is_closed and not channel.is_closed:
-                    logger.debug("%s: stop consuming queue '%s'", self, queue)
-                    await channel.basic_cancel(consume_ok.consumer_tag, timeout=self.config.timeout)
-                    await channel.close()
-            self._task_consumers = {}
+            try:
+                for queue in list(self._task_consumers.keys()):
+                    if queues is None or queue in queues:
+                        channel, consume_ok = self._task_consumers[queue]
+                        if not self.__conn.is_closed and not channel.is_closed:
+                            logger.debug("%s: stop consuming queue '%s'", self, queue)
+                            await channel.basic_cancel(consume_ok.consumer_tag, timeout=self.config.timeout)
+                            await channel.close()
+                        del self._task_consumers[queue]
+            finally:
+                if queues is None:
+                    self.__conn.remove_callback("on_lost", "consume_tasks")
+                    self.__conn.remove_callback("on_close", "consume_tasks")
+                    self._task_consumers.clear()
 
     @base.Backend.task
     async def declare_result_queue(self, task_instance: TaskInstance):
         task_id = task_instance.data.task_id
         result_ttl = task_instance.data.result_ttl
         queue = routing_key = f"result.{task_id}"
-        durable = task_instance.data.backend_extra.get("result_queue_durable")
+        durable = task_instance.data.extra.get("result_queue_durable")
         async with self._conn.channel_ctx() as channel:
             await channel.queue_declare(
                 queue,
@@ -443,7 +449,7 @@ class Backend(base.Backend):
         logger.debug("%s: push result for %s", self, task_instance)
         async with self._conn.channel_ctx() as channel:
             await channel.basic_publish(
-                self.serializer.dumps_task_result(task_result, encrypt=task_instance.data.result_encrypt),
+                self.serializer.dumps_task_result(task_instance, task_result),
                 exchange=self.config.tasks_exchange,
                 routing_key=routing_key,
                 properties=aiormq.spec.Basic.Properties(
@@ -509,7 +515,7 @@ class Backend(base.Backend):
         logger.debug("%s: put %s", self, message)
         async with self._conn.channel_ctx() as channel:
             await channel.basic_publish(
-                self.serializer.dumps(message.data, encrypt=message.encrypt),
+                self.serializer.dumps(message.data),
                 exchange=message.exchange,
                 routing_key=routing_key,
                 properties=aiormq.spec.Basic.Properties(
@@ -546,8 +552,8 @@ class Backend(base.Backend):
                     await asyncio.shield(on_message(message))
                     if ack_late:
                         await channel.basic_ack(msg.delivery.delivery_tag)
-                except Exception:
-                    logger.exception("Internal error")
+                except Exception as e:
+                    logger.exception(e)
 
             for queue in queues:
                 if queue in self._message_consumers and not self._message_consumers[queue][0].is_closed:
@@ -607,8 +613,8 @@ class Backend(base.Backend):
                     await asyncio.shield(on_event(event))
                     if ack_late:
                         await channel.basic_ack(msg.delivery.delivery_tag)
-                except Exception:
-                    logger.exception("Internal error")
+                except Exception as e:
+                    logger.exception(e)
 
                 if self._events_consumer and not self._events_consumer[0].is_closed:
                     return
