@@ -1,23 +1,23 @@
 import asyncio
 import copy
-import datetime
-import inspect
 import logging
-import sys
-import threading
-import time
+
+from contextlib import AsyncExitStack
+from contextvars import ContextVar
 from types import FunctionType, MethodType, ModuleType
-from typing import Any, Dict, Type, Union
+from typing import Any, Dict, List, Type, Union
 from uuid import UUID
 
 from arrlio import __tasks__
-from arrlio.exc import NotFoundError, TaskError, TaskNoResultError, TaskTimeoutError
+from arrlio.exc import TaskError, TaskNoResultError
 from arrlio.models import Event, Graph, Message, Task, TaskData, TaskInstance, TaskResult
+from arrlio.plugins.base import Plugin
 from arrlio.settings import Config
 from arrlio.tp import AsyncCallableT
+from roview import rodict, rolist
 
 
-logger = logging.getLogger("arrlio")
+logger = logging.getLogger("arrlio.core")
 
 
 def task(func: FunctionType = None, name: str = None, base: Type[Task] = None, **kwds):
@@ -50,86 +50,6 @@ def task(func: FunctionType = None, name: str = None, base: Type[Task] = None, *
         return wrapper
 
 
-class Executor:
-    def __str__(self):
-        return f"[{self.__class__.__name__}]"
-
-    def __repr__(self):
-        return self.__str__()
-
-    async def __call__(self, task_instance: TaskInstance) -> TaskResult:
-        task: Task = task_instance.task
-        task_data: TaskData = task_instance.data
-
-        res, exc, trb = None, None, None
-        t0 = time.monotonic()
-
-        logger.info("%s: execute task %s(%s)", self, task.name, task_data.task_id)
-
-        try:
-
-            if task.func is None:
-                raise NotFoundError(f"Task '{task.name}' not found")
-
-            kwdefaults = task.func.__kwdefaults__
-            meta: bool = kwdefaults is not None and "meta" in kwdefaults
-
-            try:
-                if inspect.iscoroutinefunction(task.func):
-                    res = await asyncio.wait_for(task_instance(meta=meta), task_data.timeout)
-                else:
-                    res = task_instance(meta=meta)
-            except asyncio.TimeoutError:
-                raise TaskTimeoutError(task_data.timeout)
-
-        except Exception as e:
-            exc_info = sys.exc_info()
-            exc, trb = exc_info[1], exc_info[2]
-            if isinstance(e, TaskTimeoutError):
-                logger.error("Task timeout for %s", task_instance)
-            else:
-                logger.exception("%s: %s", self, task_instance)
-
-        logger.info(
-            "%s: task %s(%s) done in %.2f second(s)",
-            self,
-            task.name,
-            task_data.task_id,
-            time.monotonic() - t0,
-        )
-
-        if isinstance(res, TaskResult):
-            return res
-
-        return TaskResult(res=res, exc=exc, trb=trb)
-
-
-class ThreadExecutor(Executor):
-    async def __call__(self, task_instance: TaskInstance) -> TaskResult:
-        root_loop = asyncio.get_running_loop()
-        done_ev: asyncio.Event = asyncio.Event()
-        task_result: TaskResult = None
-
-        def thread():
-            nonlocal done_ev
-            nonlocal task_result
-            loop = asyncio.new_event_loop()
-            try:
-                asyncio.set_event_loop(loop)
-                task_result = loop.run_until_complete(super(ThreadExecutor, self).__call__(task_instance))
-            finally:
-                loop.run_until_complete(loop.shutdown_asyncgens())
-                loop.close()
-                root_loop.call_soon_threadsafe(lambda: done_ev.set())
-
-        th = threading.Thread(target=thread)
-        th.start()
-
-        await done_ev.wait()
-
-        return task_result
-
-
 class App:
     """
     Args:
@@ -146,9 +66,33 @@ class App:
         self._running_tasks: Dict[UUID, asyncio.Task] = {}
         self._running_messages: Dict[UUID, asyncio.Task] = {}
         self._lock: asyncio.Lock = asyncio.Lock()
+        self._executor = config.executor()
+        self._context = ContextVar("context", default={})
+
+        self._hooks = {
+            "on_init": [],
+            "on_close": [],
+            "on_task_send": [],
+            "on_task_received": [],
+            "on_task_done": [],
+            "task_context": [],
+        }
+        self._plugins = {}
+        for plugin_cls in config.plugins:
+            plugin = plugin_cls(self)
+            self._plugins[plugin.name] = plugin
+            for k in self._hooks:
+                if getattr(plugin, k).__func__ != getattr(Plugin, k):
+                    self._hooks[k].append(getattr(plugin, k))
+
+        self._plugins_tasks = []
+        for hook in self._hooks["on_init"]:
+            task = asyncio.create_task(hook())
+            self._plugins_tasks.append(task)
+            task.add_done_callback(lambda fut: self._plugins_tasks.remove(task))
 
     def __str__(self):
-        return f"[{self.__class__.__name__}{self._backend}]"
+        return f"{self.__class__.__name__}[{self._backend}]"
 
     def __repr__(self):
         return self.__str__()
@@ -159,27 +103,74 @@ class App:
     async def __aexit__(self, exc_type, exc, tb):
         await self.close()
 
+    @property
+    def hooks(self):
+        return rolist(list(self._hooks.keys()))
+
+    @property
+    def plugins(self):
+        return rodict(self._plugins)
+
     @property
     def backend(self):
         return self._backend
 
+    @property
+    def executor(self):
+        return self._executor
+
+    @property
+    def context(self):
+        return self._context
+
     @property
     def is_closed(self) -> bool:
         return self._closed.done()
 
+    async def wait_init(self):
+        for task in self._plugins_tasks:
+            await task
+
     async def close(self):
-        await self.stop_consume_tasks()
-        await self.stop_consume_messages()
-        await self.stop_consume_events()
-        await self._backend.close()
-        self._closed.set_result(None)
+        try:
+            for task in self._plugins_tasks:
+                task.cancel()
+                logger.warning("Cancel %s", task)
+
+            await self._execute_hooks("on_close")
+            for k in self._hooks:
+                self._hooks[k].clear()
+
+            await self.stop_consume_tasks()
+            await self.stop_consume_messages()
+            await self.stop_consume_events()
+
+            for task_id, aio_task in self._running_tasks.items():
+                logger.debug("%s: cancel processing task '%s'", str(self), task_id)
+                aio_task.cancel()
+            self._running_tasks = {}
+
+            for message_id, aio_task in self._running_messages.items():
+                logger.debug("%s: cancel processing message '%s'", str(self), message_id)
+                aio_task.cancel()
+
+            await self._backend.close()
+        finally:
+            self._closed.set_result(None)
+
+    async def _execute_hooks(self, hook: str, *args, **kwds):
+        for hook_fn in self._hooks[hook]:
+            try:
+                await hook_fn(*args, **kwds)
+            except Exception as e:
+                logger.error("%s: plugin %s error: %s", str(self), hook_fn, e)
 
     async def send_task(
         self,
         task: Union[Task, str],
         args: tuple = None,
         kwds: dict = None,
-        backend_extra: dict = None,
+        extra: dict = None,
         **kwargs,
     ) -> "AsyncResult":
         """
@@ -187,7 +178,7 @@ class App:
             task (Union[~arrlio.models.Task, str]): ~arrlio.models.Task of task name.
             args (tuple, optional): Task args.
             kwds (dict, optional): Task kwds.
-            backend_extra (dict, optional): ~arrlio.models.TaskData backend_extra.
+            extra (dict, optional): ~arrlio.models.TaskData extra.
             kwargs (dict, optional): Other ~arrlio.models.TaskData arguments.
 
         Returns:
@@ -202,21 +193,23 @@ class App:
             args = ()
         if kwds is None:
             kwds = {}
-        if backend_extra is None:
-            backend_extra = {}
+        if extra is None:
+            extra = {}
 
         task_settings = self.config.task.dict(exclude_unset=True)
 
         if name in __tasks__:
             task_instance = __tasks__[name].instantiate(
-                args=args, kwds=kwds, backend_extra=backend_extra, **{**task_settings, **kwargs}
+                args=args, kwds=kwds, extra=extra, **{**task_settings, **kwargs}
             )
         else:
             task_instance = Task(None, name).instantiate(
-                args=args, kwds=kwds, backend_extra=backend_extra, **{**task_settings, **kwargs}
+                args=args, kwds=kwds, extra=extra, **{**task_settings, **kwargs}
             )
 
-        logger.info("%s: send %s", self, task_instance)
+        logger.info("%s: send %s", str(self), task_instance)
+
+        await self._execute_hooks("on_task_send", task_instance)
 
         await self._backend.send_task(task_instance)
 
@@ -247,7 +240,7 @@ class App:
         if not nodes or not roots:
             raise ValueError("Empty graph or missing roots")
 
-        logger.info("%s: send %s with args: %s and kwds: %s", self, graph, args, kwds)
+        logger.info("%s: send %s with args: %s and kwds: %s", str(self), graph, args, kwds)
 
         task_instances = {}
         for node_name, (task, node_kwds) in nodes.items():
@@ -267,7 +260,7 @@ class App:
             data.meta.update(meta or {})
             data.graph = Graph(graph.id, nodes=nodes, edges=edges, roots={root})
 
-            logger.info("%s: send %s", self, task_instances[root])
+            logger.info("%s: send %s", str(self), task_instances[root])
 
             await self._backend.send_task(task_instances[root])
 
@@ -284,10 +277,14 @@ class App:
         message_settings = self.config.message.dict(exclude_unset=True)
         message = Message(data=data, **{**message_settings, **kwds})
 
-        logger.info("%s: send %s", self, message)
+        logger.info("%s: send %s", str(self), message)
 
         await self._backend.send_message(message, routing_key=routing_key)
 
+    async def send_event(self, event: Event):
+        logger.info("%s: send %s", str(self), event)
+        await self._backend.push_event(event)
+
     async def pop_result(self, task_instance: TaskInstance):
         task_result: TaskResult = await self._backend.pop_task_result(task_instance)
         if task_result.exc:
@@ -297,58 +294,20 @@ class App:
                 raise TaskError(task_result.exc, task_result.trb)
         return task_result.res
 
-    async def consume_tasks(self):
-        if self.config.task_queues:
-            await self._backend.consume_tasks(self.config.task_queues, self._on_task)
-            logger.info("%s: consuming task queues %s", self, self.config.task_queues)
+    async def consume_tasks(self, queues: List[str] = None):
+        queues = queues or self.config.task_queues
+        if queues:
+            await self._backend.consume_tasks(queues, self._on_task)
+            logger.info("%s: consuming task queues %s", str(self), queues)
 
-    async def stop_consume_tasks(self):
+    async def stop_consume_tasks(self, queues: List[str] = None):
         async with self._lock:
-            await self._backend.stop_consume_tasks()
-            for task_id, aio_task in self._running_tasks.items():
-                logger.debug("Cancel processing task '%s'", task_id)
-                aio_task.cancel()
-            self._running_tasks = {}
-            logger.info("%s: stop consuming task queues %s", self, self.config.task_queues)
-
-    def make_event(self, event_type: str, event_data: dict = None) -> Event:
-        return Event(
-            type=event_type,
-            dt=datetime.datetime.now(tz=datetime.timezone.utc),
-            data=event_data or {},
-        )
-
-    async def on_task_received(self, task_instance: TaskInstance):
-        task_type = "task received"
-        task_data = task_instance.data
-        if (
-            task_data.events is True
-            or isinstance(task_data.events, (list, set, tuple))
-            and task_type in task_data.events
-        ):
-            event: Event = self.make_event(
-                task_type,
-                event_data={"task_id": task_instance.data.task_id},
-            )
-            await self._backend.push_event(event)
-
-    async def on_task_done(self, task_instance: TaskInstance, task_result: TaskResult):
-        task_type = "task done"
-        task_data = task_instance.data
-        if (
-            task_data.events is True
-            or isinstance(task_data.events, (list, set, tuple))
-            and task_type in task_data.events
-        ):
-            event: Event = self.make_event(
-                task_type,
-                event_data={"task_id": task_instance.data.task_id, "status": task_result.exc is None},
-            )
-            await self._backend.push_event(event)
+            await self._backend.stop_consume_tasks(queues=queues)
+            logger.info("%s: stop consuming task queues %s", str(self), self.config.task_queues)
 
     async def _on_task(self, task_instance: TaskInstance):
         try:
-            await self.on_task_received(task_instance)
+            await self._execute_hooks("on_task_received", task_instance)
 
             task_id: UUID = task_instance.data.task_id
 
@@ -377,57 +336,47 @@ class App:
             logger.exception(e)
 
     async def execute_task(self, task_instance: TaskInstance):
-        try:
-            task_data: TaskData = task_instance.data
+        async with AsyncExitStack() as stack:
+            for context in self._hooks["task_context"]:
+                await stack.enter_async_context(context(task_instance))
 
-            if task_data.thread is True:
-                executor = ThreadExecutor()
-            else:
-                executor = Executor()
+            try:
+                task_data: TaskData = task_instance.data
 
-            task_result: TaskResult = await executor(task_instance)
+                task_result: TaskResult = await self._executor(task_instance)
 
-            graph: Graph = task_data.graph
-            if graph is not None and task_result.exc is None:
-                routes = task_result.routes
-                args = (task_result.res,) or ()
-                if isinstance(routes, str):
-                    routes = [routes]
+                graph: Graph = task_data.graph
+                if graph is not None and task_result.exc is None:
+                    routes = task_result.routes
+                    args = (task_result.res,) or ()
+                    if isinstance(routes, str):
+                        routes = [routes]
 
-                root: str = next(iter(graph.roots))
-                if root in graph.edges:
-                    for node_id, node_id_routes in graph.edges[root]:
-                        if not ((routes is None and node_id_routes is None) or set(routes) & set(node_id_routes)):
-                            continue
-                        # graph: Graph = Graph(
-                        #     id=graph.id,
-                        #     nodes=graph.nodes,
-                        #     edges=graph.edges,
-                        #     roots={node_id},
-                        # )
-                        await self.send_graph(
-                            Graph(
-                                id=graph.id,
-                                nodes=graph.nodes,
-                                edges=graph.edges,
-                                roots={node_id},
-                            ),
-                            args=args,
-                            meta={"source_node": root},
-                        )
+                    root: str = next(iter(graph.roots))
+                    if root in graph.edges:
+                        for node_id, node_id_routes in graph.edges[root]:
+                            if not ((routes is None and node_id_routes is None) or set(routes) & set(node_id_routes)):
+                                continue
+                            await self.send_graph(
+                                Graph(
+                                    id=graph.id,
+                                    nodes=graph.nodes,
+                                    edges=graph.edges,
+                                    roots={node_id},
+                                ),
+                                args=args,
+                                meta={"source_node": root},
+                            )
 
-            if task_instance.data.result_return:
-                try:
+                if task_instance.data.result_return:
                     await self._backend.push_task_result(task_instance, task_result)
-                except Exception as e:
-                    logger.exception(e)
 
-            await self.on_task_done(task_instance, task_result)
+                await self._execute_hooks("on_task_done", task_instance, task_result)
 
-        except asyncio.CancelledError:
-            pass
-        except Exception as e:
-            logger.exception(e)
+            except asyncio.CancelledError:
+                pass
+            except Exception as e:
+                logger.exception(e)
 
     async def consume_messages(self, on_message: AsyncCallableT):
         if self.config.message_queues:
@@ -442,7 +391,7 @@ class App:
                             try:
                                 aio_task = asyncio.create_task(on_message(message.data))
                                 aio_task.add_done_callback(lambda *args: self._running_messages.pop(message_id, None))
-                                self._running_tasks[message_id] = aio_task
+                                self._running_messages[message_id] = aio_task
                                 await aio_task
                             finally:
                                 if not self.is_closed:
@@ -456,19 +405,16 @@ class App:
                 except Exception as e:
                     logger.exception(e)
 
-            logger.info("%s: consuming message queues %s", self, self.config.message_queues)
+            logger.info("%s: consuming message queues %s", str(self), self.config.message_queues)
             await self._backend.consume_messages(self.config.message_queues, handle)
 
     async def stop_consume_messages(self):
         async with self._lock:
             await self._backend.stop_consume_messages()
-            for message_id, aio_task in self._running_messages.items():
-                logger.debug("Cancel processing message '%s'", message_id)
-                aio_task.cancel()
             self._running_messages = {}
 
     async def consume_events(self, on_event: AsyncCallableT):
-        logger.info("%s: consuming events", self)
+        logger.info("%s: consuming events", str(self))
         await self._backend.consume_events(on_event)
 
     async def stop_consume_events(self):
